==> noodles-cram-0.8.3.crate: noodles-cram-0.8.3/src/data_container/compression_header/preservation_map/tag_ids_dictionary/builder.rs <==
use std::collections::HashMap;

use crate::{record, Record};

use super::TagIdsDictionary;

#[derive(Debug, Default)]
pub struct Builder {
    keys_indices: HashMap<Vec<record::tag::Key>, usize>,
}

impl Builder {
    pub fn update(&mut self, record: &Record) {
        let keys: Vec<_> = record.tags().iter().map(|tag| tag.key()).collect();
        let next_index = self.keys_indices.len();
        self.keys_indices.entry(keys).or_insert(next_index);
    }

    pub fn build(self) -> TagIdsDictionary {
        let mut lines: Vec<_> = self.keys_indices.into_iter().collect();
        lines.sort_by_key(|(_, index)| *index);
        let dictionary: Vec<_> = lines.into_iter().map(|(keys, _)| keys).collect();
        TagIdsDictionary::from(dictionary)
    }
}

#[cfg(test)]
mod tests {
    use noodles_bam::record::data::field::{value::Type, Value};

    use crate::record::{tag::Key, Tag};

    use super::*;

    #[test]
    fn test_from_records() {
        let mut builder = Builder::default();

        let mut record = Record::default();
        record
            .tags
            .push(Tag::new(Key::new([b'N', b'H'], Type::Int8), Value::Int8(1)));
        builder.update(&record);

        let mut record = Record::default();
        record
            .tags
            .push(Tag::new(Key::new([b'N', b'H'], Type::Int8), Value::Int8(2)));
        builder.update(&record);

        let mut record = Record::default();
        record
            .tags
            .push(Tag::new(Key::new([b'N', b'H'], Type::Int8), Value::Int8(1)));
        record.tags.push(Tag::new(
            Key::new([b'C', b'O'], Type::String),
            Value::String(String::from("noodles")),
        ));
        builder.update(&record);

        let dictionary = builder.build();

        assert_eq!(
            *dictionary,
            [
                vec![Key::new([b'N', b'H'], Type::Int8)],
                vec![
                    Key::new([b'N', b'H'], Type::Int8),
                    Key::new([b'C', b'O'], Type::String)
                ]
            ]
        )
    }
}

==> noodles-cram-0.15.0.crate: noodles-cram-0.15.0/src/data_container/compression_header/preservation_map/tag_ids_dictionary/builder.rs <==
use std::collections::HashMap;

use super::{Key, TagIdsDictionary};
use crate::Record;

#[derive(Debug, Default)]
pub struct Builder {
    keys_indices: HashMap<Vec<Key>, usize>,
}

impl Builder {
    pub fn update(&mut self, record: &Record) {
        let keys: Vec<_> = record.tags().values().map(|field| field.into()).collect();
        let next_index = self.keys_indices.len();
        self.keys_indices.entry(keys).or_insert(next_index);
    }

    pub fn build(self) -> TagIdsDictionary {
        let mut lines: Vec<_> = self.keys_indices.into_iter().collect();
        lines.sort_by_key(|(_, index)| *index);
        let dictionary: Vec<_> = lines.into_iter().map(|(keys, _)| keys).collect();
        TagIdsDictionary::from(dictionary)
    }
}

#[cfg(test)]
mod tests {
    use noodles_sam::record::data::{
        field::{value::Type, Tag, Value},
        Field,
    };

    use super::*;

    #[test]
    fn test_from_records() {
        let mut builder = Builder::default();

        let mut record = Record::default();
        record
            .tags
            .insert(Field::new(Tag::AlignmentHitCount, Value::Int8(1)));
        builder.update(&record);

        let mut record = Record::default();
        record
            .tags
            .insert(Field::new(Tag::AlignmentHitCount, Value::Int8(2)));
        builder.update(&record);

        let mut record = Record::default();
        record
            .tags
            .insert(Field::new(Tag::AlignmentHitCount, Value::Int8(1)));
        record.tags.insert(Field::new(
            Tag::Comment,
            Value::String(String::from("noodles")),
        ));
        builder.update(&record);

        let dictionary = builder.build();

        assert_eq!(
            *dictionary,
            [
                vec![Key::new(Tag::AlignmentHitCount, Type::Int8)],
                vec![
                    Key::new(Tag::AlignmentHitCount, Type::Int8),
                    Key::new(Tag::Comment, Type::String)
                ]
            ]
        )
    }
}
